#include <avr/io.h>
#include <avr/interrupt.h>

#include <util/delay.h>

#include <stdlib.h>
#include <stdio.h>

void init_pwm() {

	// Timer1
	// Init output pins
	DDRD |= (1 << PD4);
	DDRD |= (1 << PD5);

	// Non-inverting mode
	TCCR1A = (1 << COM1B1) | (1 << COM1A1);

	// Fast PWM - TOP ICR
	TCCR1A |= (1 << WGM11);
	TCCR1B = (1 << WGM12) | (1 << WGM13);

	// clk 256 from prescaler
	TCCR1B |= (1 << CS12);

	// Top for timer1
	ICR1 = 220;

	// Timer0
	DDRB |= (1 << PB3);
	DDRB |= (1 << PB4);

	// Non-inverting mode
	TCCR0A = (1 << COM0B1) | (1 << COM0A1);

	// Fast PWM - TOP 0xFF
	TCCR0A |= (1 << WGM00);

	// clk 64 from prescaler
	TCCR0B |= (1 << CS00) | (1 << CS01);

	// set default direction
	OCR1A = 0;
	OCR1B = 100;

	OCR0A = 80;
	OCR0B = 0;
}

void init_ir() {
	// Input pins for IR
	DDRA &= ~(1 << PA0);
	DDRA &= ~(1 << PA1);
}

int main(void) {

	// Init stuff
	init_pwm();
	init_ir();

	uint8_t front_ir = 0;
	uint8_t back_ir = 0;

	int val = 0;
	int dir = 1;

	while(1) {

		front_ir = PINA & 0x01;
		back_ir = PINA & 0x10;

		// Get next movement
		if (front_ir == 0x01) {
			OCR1A = 0;
			OCR1B = (170 - OCR1A);

			OCR0A = 0;
			OCR0B = (170 - OCR0A);
			dir = 1;
		} else if (back_ir == 0x01) {
			OCR1A = 170 - OCR1B;
			OCR1B = 0;

			OCR0A = 170 - OCR0B;
			OCR0B = 0;
			dir = -1;
		} else {
			if (dir == 1) {
				OCR1A = 100;
				OCR1B = 0;

				OCR0A = 120;
				OCR0B = 0;
			} else {
				OCR1A = 0;
				OCR1B = 100;

				OCR0A = 0;
				OCR0B = 120;
			}
		}

		// "random" movement
		ICR1 += (-1) * ICR1 + (val % 6) * 10;

		_delay_ms(500);

		val++;
	}
}
